﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Data
Imports System.Data.SqlClient

Imports NorthwindCacheDemo.DAL
Imports NorthwindCacheDemo.DTO

Namespace NorthwindCacheDemo.DAL.SqlClient
    ''' <summary>
    ''' Provides data access to the Northwind product catalog.
    ''' The methods in this class use System.Data.SqlClient to 
    ''' retrieve data from the Northwind database. Further, by using 
    ''' the SqlDataReader, these data access methods build transfer 
    ''' objects that are subsequently passed to consuming layers.
    ''' </summary>
    Public Class SqlCatalogProvider
        Inherits SqlProvider
        Implements ICatalogProvider
        Public Sub New()
        End Sub

        Public Function GetCategories() As IList(Of Category) Implements ICatalogProvider.GetCategories
            Dim categories As IList(Of Category) = Nothing
            Using conn As New SqlConnection(Me.ConnectionString)
                Dim cmd As New SqlCommand("SELECT * FROM [Northwind].[dbo].[Categories]", conn)
                cmd.CommandType = CommandType.Text
                conn.Open()

                categories = CreateCategoriesFromDataReader(cmd.ExecuteReader())
            End Using

            Return categories
        End Function

        Public Function GetCategory(ByVal id As Integer) As Category Implements ICatalogProvider.GetCategory
            Dim category As Category = Nothing

            Using conn As New SqlConnection(Me.ConnectionString)
                Dim cmd As New SqlCommand("SELECT * FROM [Northwind].[dbo].[Categories] AS c WHERE c.CategoryID = " & id.ToString(), conn)
                cmd.CommandType = CommandType.Text
                conn.Open()

                Dim dr As SqlDataReader = cmd.ExecuteReader()

                If (Not dr.Read()) Then
                    Return Nothing
                End If

                category = CreateCategoryFromDataReader(dr)
            End Using

            Return category
        End Function

        Public Function GetProduct(ByVal id As Integer) As Product Implements ICatalogProvider.GetProduct
            Dim product As Product = Nothing

            Using conn As New SqlConnection(Me.ConnectionString)
                Dim cmd As New SqlCommand("SELECT * FROM [Northwind].[dbo].[Products] AS p WHERE p.ProductID = " & id.ToString(), conn)
                cmd.CommandType = CommandType.Text
                conn.Open()

                Dim dr As SqlDataReader = cmd.ExecuteReader()

                If (Not dr.Read()) Then
                    Return Nothing
                End If

                product = CreateProductFromDataReader(dr)
            End Using

            Return product
        End Function

        Public Function GetProductsByCategory(ByVal id As Integer) As IList(Of Product) Implements ICatalogProvider.GetProductsByCategory
            Dim products As IList(Of Product) = Nothing

            Using conn As New SqlConnection(Me.ConnectionString)
                Dim cmd As New SqlCommand("SELECT * FROM [Northwind].[dbo].[Products] AS p WHERE p.CategoryID = " & id.ToString(), conn)
                cmd.CommandType = CommandType.Text
                conn.Open()

                products = CreateProductsFromDataReader(cmd.ExecuteReader())
            End Using

            Return products
        End Function

        Public Function GetPopularProducts() As IList(Of Product) Implements ICatalogProvider.GetPopularProducts
            Dim commandText As String = "SELECT * " & ControlChars.CrLf & "                FROM [Northwind].[dbo].[Products] AS p" & ControlChars.CrLf & "                WHERE p.ProductID IN" & ControlChars.CrLf & "	                (SELECT TopProducts.ProductID " & ControlChars.CrLf & "	                FROM" & ControlChars.CrLf & "		                (SELECT TOP 10 items.ProductID AS ProductID, COUNT(items.ProductID) AS ProductCount" & ControlChars.CrLf & "		                FROM [Northwind].[dbo].[Order Details] AS items" & ControlChars.CrLf & "		                GROUP BY items.ProductID" & ControlChars.CrLf & "		                ORDER BY ProductCount DESC) AS TopProducts)"

            Dim products As IList(Of Product) = Nothing

            Using conn As New SqlConnection(Me.ConnectionString)
                Dim cmd As New SqlCommand(commandText, conn)
                cmd.CommandType = CommandType.Text
                conn.Open()

                products = CreateProductsFromDataReader(cmd.ExecuteReader())
            End Using

            Return products
        End Function

        Public Function GetPopularProductsByCategory(ByVal id As Integer) As IList(Of Product) Implements ICatalogProvider.GetPopularProductsByCategory
            Dim commandText As String = "SELECT * " & ControlChars.CrLf & "                FROM [Northwind].[dbo].[Products] AS p" & ControlChars.CrLf & "                WHERE p.ProductID IN" & ControlChars.CrLf & "	                (SELECT TopProducts.ProductID " & ControlChars.CrLf & "	                FROM" & ControlChars.CrLf & "		                (SELECT TOP 3 items.ProductID AS ProductID, COUNT(items.ProductID) AS ProductCount" & ControlChars.CrLf & "		                FROM [Northwind].[dbo].[Order Details] AS items" & ControlChars.CrLf & "		                WHERE items.ProductID IN " & ControlChars.CrLf & "			                (SELECT categoryProducts.ProductID" & ControlChars.CrLf & "			                FROM [Northwind].[dbo].[Products] AS categoryProducts" & ControlChars.CrLf & "			                WHERE categoryProducts.CategoryID = " & id.ToString() & ")" & ControlChars.CrLf & "		                GROUP BY items.ProductID" & ControlChars.CrLf & "		                ORDER BY ProductCount DESC) AS TopProducts)"

            Dim products As IList(Of Product) = Nothing

            Using conn As New SqlConnection(Me.ConnectionString)
                Dim cmd As New SqlCommand(commandText, conn)
                cmd.CommandType = CommandType.Text
                conn.Open()

                products = CreateProductsFromDataReader(cmd.ExecuteReader())
            End Using

            Return products
        End Function

        Public Function GetAlsoPurchasedProducts(ByVal id As Integer) As IList(Of Product) Implements ICatalogProvider.GetAlsoPurchasedProducts
            Dim idString As String = id.ToString()

            Dim commandText As String = "SELECT *" & ControlChars.CrLf & "                FROM [Northwind].[dbo].[Products] AS p" & ControlChars.CrLf & "                WHERE p.ProductID IN " & ControlChars.CrLf & "	                (SELECT TopProducts.ProductID" & ControlChars.CrLf & "	                FROM" & ControlChars.CrLf & "		                (SELECT TOP 5 i.ProductID AS ProductID, COUNT(i.ProductID) AS ProductCount" & ControlChars.CrLf & "		                FROM [Northwind].[dbo].[Order Details] as i" & ControlChars.CrLf & "		                WHERE i.OrderID IN" & ControlChars.CrLf & "			                (SELECT d.OrderID" & ControlChars.CrLf & "			                FROM [Northwind].[dbo].[Order Details] as d" & ControlChars.CrLf & "			                WHERE d.ProductID = " & idString & ")" & ControlChars.CrLf & "		                GROUP BY i.ProductID" & ControlChars.CrLf & "		                HAVING i.ProductID <> " & idString & ControlChars.CrLf & "		                ORDER BY ProductCount DESC) AS TopProducts)"

            Dim products As IList(Of Product) = Nothing

            Using conn As New SqlConnection(Me.ConnectionString)
                Dim cmd As New SqlCommand(commandText, conn)
                cmd.CommandType = CommandType.Text
                conn.Open()

                products = CreateProductsFromDataReader(cmd.ExecuteReader())
            End Using

            Return products
        End Function

        Private Function CreateCategoriesFromDataReader(ByVal dr As SqlDataReader) As IList(Of Category)
            Dim categories As IList(Of Category) = New List(Of Category)()
            Do While dr.Read()
                categories.Add(CreateCategoryFromDataReader(dr))
            Loop
            Return If((categories.Count > 0), categories, Nothing)
        End Function

        Private Function CreateCategoryFromDataReader(ByVal dr As SqlDataReader) As Category
            Return New Category() With _
            { _
                .CategoryID = Convert.ToInt32(dr("CategoryID")), _
                .Name = Convert.ToString(dr("CategoryName")), _
                .Description = Convert.ToString(dr("Description")) _
            }
        End Function

        Private Function CreateProductsFromDataReader(ByVal dr As SqlDataReader) As IList(Of Product)
            Dim products As IList(Of Product) = New List(Of Product)()
            Do While dr.Read()
                products.Add(CreateProductFromDataReader(dr))
            Loop
            Return If((products.Count > 0), products, Nothing)
        End Function

        Private Function CreateProductFromDataReader(ByVal dr As SqlDataReader) As Product
            Return New Product() With _
            { _
                .UnitPrice = Convert.ToDecimal(dr("UnitPrice")), _
                .QuantityPerUnit = Convert.ToString(dr("QuantityPerUnit")), _
                .Name = Convert.ToString(dr("ProductName")), _
                .CategoryID = Convert.ToInt32(dr("CategoryID")), _
                .ProductID = Convert.ToInt32(dr("ProductID")) _
            }
        End Function
    End Class
End Namespace
